@@ -1,5 +1,3 @@
-
-
 try:
     import pickle
 except ImportError:
@@ -8,6 +6,7 @@ except ImportError:
 import astropy.io.fits as fits # this should be specific to ddosa!
 
 import tarfile
+import traceback
 import glob
 import copy
 import gzip
@@ -19,6 +18,7 @@ import subprocess
 import time
 import socket
 import yaml as yaml
+import random
 
 
 import dataanalysis
@@ -30,6 +30,7 @@ from dataanalysis import analysisfactory
 from dataanalysis import hashtools
 from dataanalysis.printhook import log
 from dataanalysis.caches import backends
+#from dataanalysis.core import DataFile, map_nested_structure, flatten_nested_structure
 
 global_readonly_caches=False
 
@@ -62,6 +63,21 @@ class Cache(object):
     filebackend=backends.FileBackend()
 
     can_url_to_cache=True
+
+    def approved_hashe(self, hashe):
+        return True
+
+    def approved_read_cache(self, obj):
+        if not any([isinstance(self,c) for c in obj.read_caches]):
+            log("cache "+repr(self)+" should not be read by this analysis, allowed: "+repr(obj.read_caches))
+            return False
+        return True
+    
+    def approved_write_cache(self, obj):
+        if not any([isinstance(self,c) for c in obj.write_caches]):
+            log("cache "+repr(self)+" should not be read by this analysis, allowed: "+repr(obj.read_caches))
+            return False
+        return True
  
     def reset(self):
         log("resetting cache PLACEHOLDER",self)
@@ -126,6 +142,8 @@ class Cache(object):
         pass
         #if self.parent is not None:
         #    self.parent.runtime_unregister(obj)
+
+    _da_disable_parent = False
     
     def restore_from_parent(self,hashe,obj,rc=None):
         if self.parent is None:
@@ -133,12 +151,17 @@ class Cache(object):
             return None
         
         log(self,"there is a parent available to call for:",self.parent)
-        from_parent=self.parent.restore(hashe,obj,rc)
-            
-        if from_parent is not None:
-            log("storing what restored from parent") 
-# TODO: special case when object was cached and not available in tcurrent dir
-            self.store(hashe,obj)
+
+        if not self._da_disable_parent:
+            from_parent=self.parent.restore(hashe,obj,rc)
+                
+            if from_parent is not None:
+                log("storing what restored from parent? NO!") 
+                # TODO: special case when object was cached and not available in tcurrent dir
+               # self.store(hashe,obj)
+        else:
+            log("following parent is disabled in this restore request") 
+            from_parent = None
 
         return from_parent
 
@@ -151,24 +174,25 @@ class Cache(object):
         return self.parent.store(hashe,obj)
 
 
-    def load_content(self,hashe,c):
-        cached_path=self.construct_cached_file_path(hashe,c)
+    def load_content(self,hashe, obj, cached_path=None):
+        if cached_path is None:
+            cached_path = self.construct_cached_file_path(hashe, obj)
                     
-        log("restoring from",cached_path+"/cache.pickle.gz")
+        log("load_content is restoring from",cached_path+"/cache.pickle.gz", level="top")
 
         try:
-            log("loading from pickle")
+            log("trying loading from pickle")
             content=pickle.load(self.filebackend.open(cached_path+"/cache.pickle.gz",gz=True))
             log("done loading from pickle")
             return content
         except TypeError as e:
             log("typeerror! "+repr(e))
             raise
-        except (IOError,pickle.UnpicklingError) as e:
-            log("problem loading cache! corrupt cache!")
+        except (IOError, pickle.UnpicklingError) as e:
+            log("problem loading cache! corrupt cache!", e)
             raise
-        except Exception:
-            log("problem loading cache! corrupt cache!")
+        except Exception as e:
+            log("problem loading cache! corrupt cache!", e)
             raise
 
     def restore_file(self,origin,dest,obj,hashe):
@@ -279,7 +303,7 @@ class Cache(object):
     def restore_datafile(self, a, b, cached_path, restore_config, obj, hashe, add_keys, remove_keys):
         log("requested to restore DataFile", b, "mode", restore_config['datafile_restore_mode'], '{log:top}')
 
-        prefix = restore_config['datafile_target_dir']
+        prefix = restore_config.get('datafile_target_dir', None)
         if prefix is not None:
             prefix = prefix + "/"
             try:
@@ -291,9 +315,9 @@ class Cache(object):
 
         try:
             if os.path.basename(b.path).endswith(".gz"):
-                stored_filename = cached_path + os.path.basename(b.path)  # just by name? # gzip optional
+                stored_filename = os.path.join(cached_path, os.path.basename(b.path))  # just by name? # gzip optional
             else:
-                stored_filename = cached_path + os.path.basename(b.path) + ".gz"  # just by name? # gzip optional
+                stored_filename = os.path.join(cached_path, os.path.basename(b.path) + ".gz")  # just by name? # gzip optional
             log("stored filename:", stored_filename)
         except Exception as e:
             log("wat", e)
@@ -312,7 +336,7 @@ class Cache(object):
                     # just reproduce?
                     return None
 
-                log("stored file:", stored_filename, "will restore as", prefix, b.path, level='top')
+                log("stored file:", stored_filename, "will restore in prefix", prefix, "as", b.path, level='top')
 
                 b.restore_stats, restored_file = self.restore_file(stored_filename, prefix + os.path.basename(b.path),
                                                                    obj, hashe)
@@ -392,13 +416,15 @@ class Cache(object):
 
         return restore_config
 
-    def restore(self,hashe,obj,restore_config=None):
+
+    #####
+    def restore(self, hashe, obj, restore_config=None):
         # check if updated
         if obj.run_for_hashe or obj.mutating:
             return 
 
-        if not any([isinstance(self,c) for c in obj.read_caches]):
-            log("cache "+repr(self)+" should not be read by this analysis, allowed: "+repr(obj.read_caches))
+        if not self.approved_read_cache(obj):
+            log("cache", self, "not approved_read_cache, will restore_from_parent", level="top")
             from_parent=self.restore_from_parent(hashe,obj,restore_config)
             return from_parent
 
@@ -414,18 +440,28 @@ class Cache(object):
 
         if c is None:
             log("restore failed: passing to parent",self.parent)
-            return self.restore_from_parent(hashe,obj,restore_config)
+            log("cache", self, "did not find, will restore_from_parent", level="top")
+            return self.restore_from_parent(hashe, obj, restore_config)
 
         log("requested to restore cache")
         cached_path=self.construct_cached_file_path(hashe,obj)
-        log("cached path:",cached_path)
+        log(self, "in restore, cached path:",cached_path)
 
+        return self.restore_from_dir(cached_path, hashe, obj, restore_config)
+
+
+    def restore_from_dir(self, cached_path, hashe, obj, restore_config):
         obj._da_cache_path_root=cached_path
         obj._da_cached_pathes=[cached_path]
 
+        #c = content
+
+        #if c is None:
+        #    c = self.load_content(hashe, None, cached_path)
+
         try:
-            c=self.load_content(hashe,c)
-            log("load content returns:",c)
+            c = self.load_content(hashe, obj, cached_path) # why do we load it twice?..
+            log("load content returns:",)
         except Exception as e:
             log("can not load content from cache, while cache record exists! inconsistent cache!") #???
             #raise Exception("can not copy from from cache, while cache record exists! inconsistent cache!") # ???
@@ -439,14 +475,15 @@ class Cache(object):
             log("established cache record: will not acutally restore",level="cache")
             return True
 
-        if isinstance(c,dict):
+        if not isinstance(c, dict):
+            raise Exception("content from cache is not dict! "+str(c))
+        else:
             add_keys=[]
             remove_keys=[]
 
-            from dataanalysis.core import map_nested_structure
 
             def datafile_restore_mapper(k, b):
-                log("processing structure entry",k,b)
+                log("datafile_restore_mapper processing structure entry",k,b)
                 if is_datafile(b):
                     if len(k)==1:
                         a=k[0]
@@ -456,6 +493,10 @@ class Cache(object):
                     self.restore_datafile(a, b, cached_path, restore_config, obj, hashe, add_keys, remove_keys)
                 return b
 
+
+            from dataanalysis.core import DataFile, map_nested_structure, flatten_nested_structure
+                
+            log("will map_nested_structure")
             map_nested_structure(c, datafile_restore_mapper)
 
             for k, i in add_keys:
@@ -495,7 +536,6 @@ class Cache(object):
 
             log(self,"restore returning True")
             return True
-        raise Exception("content from cache is not dict! "+str(c))
 
     # TODO: clear the directory!
     def assemble_blob(self,hashe,obj):
@@ -506,10 +546,21 @@ class Cache(object):
                 tar.add(name)
 
         tar.close()
-        return open("tmp.tgz", encoding="latin-1")
+        return open("tmp.tgz", "rb")
+    
+    def restore_from_blob(self, blob_fn, hashe, obj, restore_config):
+        restored_dir = f"restored-blob-{time.time()}-{random.randint(1, int(1e10)):10d}"
+
+        if os.path.exists(restored_dir):
+            raise RuntimeError("temp restore dir exists before creating, impossible: {restored_dir}!")
+
+        with tarfile.open(blob_fn, "r:gz") as tar:
+            tar.extractall(restored_dir)
+
+        return self.restore_from_dir(restored_dir+"/blob", hashe, obj, restore_config)
+
 
     def adopt_datafiles(self,content):
-        from dataanalysis.core import DataFile, map_nested_structure  # very delayed import
 
         extra_content={}
         remove_keys=[]
@@ -525,7 +576,8 @@ class Cache(object):
 
             return adopted_b
 
-        content=map_nested_structure(content,mapping_adoption)
+        from dataanalysis.core import DataFile, map_nested_structure, flatten_nested_structure
+        content = map_nested_structure(content,mapping_adoption)
 
         if len(extra_content)>0:
             log("extra content:",extra_content)
@@ -630,7 +682,6 @@ class Cache(object):
         else:
             log('object has no alias')
 
-        from dataanalysis.core import map_nested_structure, flatten_nested_structure
 
         if isinstance(content, dict):
             #for a, b in content.items():
@@ -654,6 +705,7 @@ class Cache(object):
                          'stats': b.store_stats, 'operation': 'store'})
                 return b
 
+            from dataanalysis.core import DataFile, map_nested_structure, flatten_nested_structure
             mapped=map_nested_structure(content,datafile_mapper)
             log("mapped structure (%i):"%len(mapped))
             for k,v in flatten_nested_structure(mapped, lambda x,y:(x,y)):
@@ -663,6 +715,8 @@ class Cache(object):
         return content
 
     def store_object_content(self,hashe,obj):
+        log(self, "store object content", obj,'{log:top}')
+
         if not self.filebackend.exists(self.filecacheroot):
             self.filebackend.makedirs(self.filecacheroot)
 
@@ -675,9 +729,9 @@ class Cache(object):
 
         return self.store_to_directory(hashe, obj, cached_path)
 
-    def store(self,hashe,obj):
+    def store(self, hashe, obj):
         if obj.run_for_hashe or obj.mutating:
-            return 
+            return
         
         if self.readonly_cache:
             return self.store_to_parent(hashe,obj)
@@ -692,8 +746,8 @@ class Cache(object):
         else:
             log("object",obj,"is cached, storing")
         
-        if any([isinstance(self,c) for c in obj.write_caches]):
-            log("storing:", hashe)
+        if self.approved_write_cache(obj):
+            log(self, "storing:", hashe, level="top")
 
             content=self.store_object_content(hashe,obj)
 
@@ -716,7 +770,10 @@ class Cache(object):
         self.save()
         log("now entries",len(self.cache))
 
-    def construct_cached_file_path(self,hashe,obj):
+    def construct_cached_file_path(self,hashe, obj=None):
+        if obj is not None:
+            log(f"warning: provided obj {obj} for construct_cached_file_path, but it is not used")
+
         log("requested default cached file path")
 
         def hash_to_path(hashe):
@@ -736,7 +793,6 @@ class Cache(object):
 
         return self.filecacheroot+"/"+hash_to_path2(hashe)+"/" # choose to avoid overlapp
 
-        #return self.filecacheroot+"/"+hashe+"/"+os.path.basename(datafile.path)
 
     def save(self,target=None):
         if target is None:
@@ -908,7 +964,7 @@ class TransientCache(Cache): #d
     def store_to_parent(self,hashe,obj):
         return
 
-    def store(self,hashe,obj):
+    def store(self, hashe, obj):
        # return # problem with files
 
         log("storing in memory cache:",hashe)
@@ -942,7 +998,7 @@ class CacheIndex(Cache):
         cached_path=self.construct_cached_file_path(hashe,None)
 
         if os.path.exists(cached_path+"/cache.pickle.gz"):
-            log("found cache file:",cached_path+"/cache.pickle.gz")
+            log(self, "found cache file:",cached_path+"/cache.pickle.gz")
             try:
                 return self.load_content(hashe,None)
             except Exception as e:
@@ -967,7 +1023,7 @@ class CacheNoIndex(Cache):
 
         cached_path=self.construct_cached_file_path(hashe,None)
         if self.filebackend.exists(cached_path+"/cache.pickle.gz"):
-            log("found cache file:", cached_path+"/cache.pickle.gz", level='top')
+            log(self, "found cache file:", cached_path+"/cache.pickle.gz", level='top')
             try:
                 return self.load_content(hashe,None)
             except Exception as e:
@@ -981,6 +1037,45 @@ class CacheNoIndex(Cache):
         #raise Exception("please write to index!")
         return
 
+class CacheBlob(Cache):
+    def deposit_blob(self, hashe, blob_stream):  # 
+        raise NotImplementedError
+
+    def retrieve_blob(self, hashe): # returns stream
+        raise NotImplementedError
+
+    def store(self, hashe, obj):
+        if not self.approved_hashe(hashe) or not self.approved_write_cache(obj):
+            return self.store_to_parent(hashe, obj)
+
+        log("\033[33mtrying to store blob\033[0m", level="top")
+        blob = self.assemble_blob(hashe, obj)
+        blob.seek(0)
+
+        self.deposit_blob(hashe, blob)
+        log("after deposit stacked factory assumptions:", obj.factory.factory_assumptions_stacked, level="top")
+
+    def restore(self, hashe, obj, rc=None):
+        if not self.approved_hashe(hashe) or not self.approved_read_cache(obj):
+            return self.restore_from_parent(hashe, obj, rc)
+
+        print("\033[33mtrying to restore from blob\033[0m")
+
+        try:
+            blob_stream = self.retrieve_blob(hashe)
+        except Exception as e:
+            print("problem retrieving blob:", e, "returning None")
+            return
+        
+        if blob_stream is not None:
+            blob = blob_stream.read()
+            print("\033[33mrestored blob\033[0m: blob of", len(blob)/1024/1024, "kb")
+
+            cached_blob_fn = f"restored-blob-{random.randint(1, int(1e10)):10d}.tgz"
+
+            open(cached_blob_fn, "wb").write(blob)
+
+            return self.restore_from_blob(cached_blob_fn, hashe, obj, rc)
 
 class MemCacheIRODS(CacheNoIndex):
     can_url_to_cache=False
