@@ -1,5 +1,6 @@
 import time
 import traceback
+import yaml
 
 
 import os
@@ -44,13 +45,26 @@ class QueueCache(SelectivelyDelegatingCache):
         task_data = dict(
             object_identity=obj.get_identity().serialize(),
         )
-        r=self.queue.put(
-            task_data,
-            submission_data=dict(
-                callbacks=obj.callbacks,
-                request_origin="undefined",
-            ),
-        )
+
+        r = None
+        for i in range(20):
+            try:
+                r=self.queue.put(
+                    task_data,
+                    submission_data=dict(
+                        callbacks=obj.callbacks,
+                        request_origin="undefined",
+                    ),
+                )
+                if getattr(r, 'status_code', 200) != 200:
+                    raise Exception(f"problematic response from put task {r}")
+                break
+            except Exception as e:
+                log("problem putting task:", e)
+                time.sleep(2)
+
+        if r is None:
+            raise Exception(f"unable to put task in queue: {task_data}")
         
         log(self,"\033[31mdelegated",obj,"\033[0m with state", r['state'], level="top")
 
@@ -154,10 +168,19 @@ class QueueCacheWorker(object):
     def run_once(self):
         self.run_all()
 
-    def run_all(self, limit_tasks=1, limit_time_seconds=None, wait=10):
+    def set_worker_knowledge(self, w):
+        self._worker_knowledge = w
+
+    @property
+    def worker_knowledge(self):
+        return getattr(self, "_worker_knowledge", {})
+
+    def run_all(self, limit_tasks=1, limit_time_seconds=0, wait=10):
         log_logstash("worker", message="worker starting", worker_event="starting")
         worker_tasks=0
 
+        print('worker_knowledge:', self.worker_knowledge)
+
         worker_t0 = time.time()
 
         try:
@@ -172,7 +195,7 @@ class QueueCacheWorker(object):
             worker_tasks+=1
 
             if limit_tasks is not None and limit_tasks>0 and worker_tasks>limit_tasks:
-                log(f"\033[31mstopping worker, it completed {worker_tasks} > {limit_tasks} seconds\033[0m")
+                log(f"\033[31mstopping worker, it completed {worker_tasks} > {limit_tasks} tasks\033[0m")
                 break
 
             worker_age_seconds = time.time() - worker_t0
@@ -184,8 +207,9 @@ class QueueCacheWorker(object):
             try:
                 log("trying to get a task from", self.queue)
                 print("trying to get a task from", self.queue)
-                task=self.queue.get()
-                log("got this:",task)
+                task=self.queue.get(worker_knowledge=self.worker_knowledge)
+                print("\033[031mgot task:", task, "\033[0m")
+                print("\033[031mgot task dict:", task.task_data, "\033[0m")
                 log_logstash("worker",message="worker taking task",origin="dda_worker",worker_event="taking_task",target=task.task_data['object_identity']['factory_name'])
             except dqueue.TaskStolen:
                 time.sleep(wait)
@@ -199,7 +223,7 @@ class QueueCacheWorker(object):
             except dqueue.TaskStolen as e:
                 log("task stolen, whatever",e)
             except da.AnalysisDelegatedException as delegation_exception:
-                log("found delegated dependencies:", delegation_exception.delegation_states,level='top')
+                log("found delegated dependencies:", da.repr_short(delegation_exception.delegation_states), level='top')
                 task_dependencies = [d['task_data'] for d in delegation_exception.delegation_states]
                 #locked_task=dqueue.Task.from_file(self.queue.put(task.task_data)['fn'])
                 #assert task.filename_key == locked_task.filename_key
@@ -274,6 +298,7 @@ def main():
     parser.add_argument('-w', dest='watch', type=int, help='...', default=0)
     parser.add_argument('-W', dest='watch_closely', type=int, help='...', default=0)
     parser.add_argument('-d', dest='delay', type=int, help='...', default=10)
+    parser.add_argument('-k', dest='worker_knowledge_yaml', type=str, help='...', default=None)
 
     args=parser.parse_args()
 
@@ -281,8 +306,13 @@ def main():
         dataanalysis.printhook.global_permissive_output=True
         da.debug_output()
 
+
+
     qcworker = QueueCacheWorker(args.queue)
 
+    if args.worker_knowledge_yaml is not None:
+        qcworker.set_worker_knowledge(yaml.load(open(args.worker_knowledge_yaml), Loader=yaml.FullLoader))
+
     if args.watch_closely > 0:
         while True:
             log(qcworker.queue_status())
