@@ -1,4 +1,5 @@
 import datetime
+import json
 
 try:
     import urllib.parse
@@ -16,18 +17,24 @@ class CallbackHook(object):
         level, obj=args
         message=kwargs['message']
 
+        log("callback class:", self, "will send message \"", message, "\" for object", obj, "to callbacks ", obj.callbacks, level='callback')
         for callback_url in obj.callbacks:
             callback_filter=default_callback_filter
             if isinstance(callback_url,tuple):
                 callback_url,callback_filter_name=callback_url #
                 callback_filter=globals()[callback_filter_name]
 
-            callback_class=callback_filter
-            log("callback class:",callback_class,level='callback')
-            callback=callback_class(callback_url)
-            log("processing callback url", callback_url, callback)
+            callback_class = callback_filter
+            log("callback class:", callback_class, level='callback')
+            callback = callback_class(callback_url)
+            log("processing callback url", callback_url, callback, level='callback')
 
-            r = callback.process_callback(level=level,obj=obj,message=message,data=kwargs)
+            r = callback.process_callback(
+                    level=level,
+                    obj=obj,
+                    message=message,
+                    data=kwargs
+                )
 
             if r is not None:
                 object_data=callback.extract_data(obj)
@@ -115,6 +122,9 @@ class Callback(object):
         params.update(object_data)
         params['action'] = data.get('state', 'progress')
 
+        for k,v in params.items():
+            log(f"callback key {k} size {len(json.dumps(v))}", level='callback')
+
         if self.url.startswith("file://"):
             fn=self.url[len("file://"):]
             with open(fn,'a') as f:
@@ -133,7 +143,7 @@ class Callback(object):
                 log_hook("callback",obj,message="callback succeeded",callback_url=self.url,callback_params=self.url_params,action_params=params,callback_response_content=r)
                 return r
             except requests.ConnectionError as e:
-                log("callback failed",self.url,params,":",e,level="callback")
+                log("callback failed", self.url,params,":",e,level="callback")
                 log_hook("callback",obj,message="callback failed!",callback_exception=repr(e),callback_url=self.url,callback_params=self.url_params,action_params=params)
                 return "callback failed: " + repr(e)
         else:
